package org.glassfish.devtest.pooling.monitor.client;

import java.rmi.RemoteException;
import javax.ejb.EJB;
import org.glassfish.devtest.pooling.monitor.ejb.SimpleSessionBeanRemote;
import com.sun.ejte.ccl.reporter.SimpleReporterAdapter;
import java.io.IOException;
import org.glassfish.devtest.pooling.monitor.client.utils.Constants;
import org.glassfish.devtest.pooling.monitor.client.utils.JDBCMonitorData;
import org.glassfish.devtest.pooling.monitor.client.utils.RestHelper;

public class Client {

    private static @EJB
    SimpleSessionBeanRemote remote;
    private static SimpleReporterAdapter stat = new SimpleReporterAdapter("appserv-tests");
    String[] args;
    String poolname;
    boolean debug;
    boolean pooling;
    int minPoolSize;
    int maxPoolSize;
    int resize;
    int idleTimeout;
    boolean connectionLeakReclaim;
    int connectionLeakTimeout;
    static int LOOP = 5;
    String updateParam = "";
    int seq = 0;
    private static JDBCMonitorData expectedMonitorData;
    RestHelper rest = null;
    private static int GET_CON_COUNT = 0;
    private static int CLOSE_CON_COUNT = 0;

    public static void main(String[] args) {
        Client client = new Client(args);
        client.setUp();
        client.test();
        client.tearDown();
    }

    public Client(String[] args) {
        this.args = args;
    }

    public void setUp() {
        stat.addDescription("jdbc-pooling-monitor");
        parse();
        update();
    }

    private void parse() throws RuntimeException {
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < args.length; i++) {
            String[] param = args[i].split("=");

            if (param[0].equals(Constants.NAME)) {
                poolname = param[1];
                continue;
            } else if (param[0].equals(Constants.DEBUG)) {
                debug = Boolean.parseBoolean(param[1]);
                continue;
            } else if (param[0].equals(Constants.POOLING)) {
                pooling = Boolean.parseBoolean(param[1]);
            } else if (param[0].equals(Constants.STEADY_POOL_SIZE)) {
                minPoolSize = Integer.parseInt(param[1]);
            } else if (param[0].equals(Constants.MAX_POOL_SIZE)) {
                maxPoolSize = Integer.parseInt(param[1]);
            } else if (param[0].equals(Constants.POOL_RESIZE_QUANTITY)) {
                resize = Integer.parseInt(param[1]);
            } else if (param[0].equals(Constants.IDLE_TIMEOUT_IN_SECONDS)) {
                idleTimeout = Integer.parseInt(param[1]);
            } else if (param[0].equals(Constants.CONNECTION_LEAK_RECLAIM)) {
                connectionLeakReclaim = Boolean.parseBoolean(param[1]);
            } else if (param[0].equals(Constants.CONNECTION_LEAK_TIMEOUT_IN_SECONDS)) {
                connectionLeakTimeout = Integer.parseInt(param[1]);
            }

            if (sb.length() != 0) {
                sb.append("&");
            }
            sb.append(args[i]);
        }
        updateParam = sb.toString();
        println("updateParam: " + updateParam);
    }

    private void update() {
        update(this.updateParam);
    }

    private void update(String updateParam) {
        rest = new RestHelper(poolname);
        try {
            rest.updatePoolSetting(updateParam, debug);
        } catch (IOException ex) {
            throw new RuntimeException(ex);
        }
    }

    public void test() {
        String test = "TEST: Reconfiguration after getConnection";
        testGetConnection(test, 3, false);
        String reconfigParam = Constants.STEADY_POOL_SIZE+"=9";
        update(reconfigParam);
        try {
            println(rest.monitor().toString());
        } catch (IOException ex) {
            ex.printStackTrace();
        }
    }

    public void resetTestCount() {
        GET_CON_COUNT = 0;
        CLOSE_CON_COUNT = 0;
    }

    public void test1() {
        String test = "TEST1: Get and Close Connection (NoTransaction)";
        println(test);
        testGetConnection(test, 3, false);
        testCloseConnection(test, 3, false);
        testGetConnection(test, 3, false);
        testCloseConnection(test, 3, false);
        resetTestCount();
    }

    public void test2() {
        String test = "TEST2: Get and Close Connection (Transaction)";
        println(test);
        begin();
        testGetConnection(test, 2, true);
        testCloseConnection(test, 2, true);
        testGetConnection(test, 2, true);
        testCloseConnection(test, 2, true);
        commit(test);
        resetTestCount();
    }

    public void test3() {
        String test = "TEST3: Mark Connection As Bad (NoTransaction)";
        println(test);
        testGetConnection(test, 2, false);
        testMarkConnectionAsBad(test, 2, false);
        resetTestCount();
    }

    public void test4() {
        String test = "TEST4: Connection Leak (NoTransaction)";
        println(test);
        testGetConnection(test, 2, false);
        testDestroyLeakConnection(test, 2, false);
        resetTestCount();
    }

    private void begin() {
        println("UserTransaction.begin()");
        try {
            remote.beginForUT();
        } catch (RemoteException ex) {
            ex.printStackTrace();
        }
    }

    private void commit(String test) {
        println("UserTransaction.commit()");
        String key = test + ": COMMIT";
        try {
            remote.commitForUT();
        } catch (RemoteException ex) {
            ex.printStackTrace();
        }

        try {
            validate(key, getExpectedMonitorData());
        } catch (IOException ex) {
            ex.printStackTrace();
        }
    }

    private void rollback(String test) {
        println("UserTransaction.rollback()");
        String key = test + ": COMMIT";
        try {
            remote.rollbackForUT();
        } catch (RemoteException ex) {
            ex.printStackTrace();
        }

        try {
            validate(key, getExpectedMonitorData());
        } catch (IOException ex) {
            ex.printStackTrace();
        }
    }

    private void testGetConnection(String test, int NumOfCon, boolean tran) {
        for (int i = 0; i < NumOfCon; i++) {
            println("### Get Connection " + (++GET_CON_COUNT) + " ###");
            String key = test + ": GET_CONNECTION: " + GET_CON_COUNT;
            try {
                remote.getConnection();
                if (tran) {
                    if (i == 0) {
                        processEvent(EVENT.GET_CONNECTION);
                    } else {
                        if(!remote.isSharedConnections()){
                            throw new RuntimeException("Not Sharable Connections!");
                        }
                        processEvent(EVENT.GET_SHARED_CONNECTION);
                    }
                } else {
                    processEvent(EVENT.GET_CONNECTION);
                }
                validate(key, getExpectedMonitorData());
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }
    }

    private void testCloseConnection(String test, int NumOfCon, boolean tran) {
        JDBCMonitorData original = (JDBCMonitorData) getExpectedMonitorData().clone();
        for (int i = NumOfCon - 1; i >= 0; i--) {
            println("### Close Connection " + (++CLOSE_CON_COUNT) + " ###");
            String key = test + ": CLOSE_CONNECTION: " + CLOSE_CON_COUNT;
            try {
                remote.close(i);
                if (tran) {
                    if (i == NumOfCon - 1) {
                        processEvent(EVENT.CLOSE_CONNECTION);
                    } else {
                        processEvent(EVENT.CLOSE_SHARED_CONNECTION);
                    }
                    original.numconnreleased++;
                    validate(key, original);
                } else {
                    processEvent(EVENT.CLOSE_CONNECTION);
                    validate(key, getExpectedMonitorData());
                }
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }
    }

    private void testMarkConnectionAsBad(String test, int NumOfCon, boolean tran) {
        JDBCMonitorData original = (JDBCMonitorData) getExpectedMonitorData().clone();
        for (int i = NumOfCon - 1; i >= 0; i--) {
            println("### Mark as Bad and Close " + (++CLOSE_CON_COUNT) + " ###");
            String key = test + ": MARK_AS_BAD_AND_CLOSE: " + CLOSE_CON_COUNT;
            try {
                remote.markConnectionAsBad(i);
                remote.close(i);
                if (tran) {
                    if (i == NumOfCon - 1) {
                        processEvent(EVENT.MARK_CONNECTION_AS_BAD);
                    } else {
                        processEvent(EVENT.CLOSE_SHARED_CONNECTION);
                    }
                    original.numconnreleased++;
                    validate(key, original);
                } else {
                    processEvent(EVENT.MARK_CONNECTION_AS_BAD);
                    validate(key, getExpectedMonitorData());
                }
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }
    }

    private void testDestroyLeakConnection(String test, int NumOfCon, boolean tran) {
        JDBCMonitorData original = (JDBCMonitorData) getExpectedMonitorData().clone();
        println("### Detect Leak Connection and Destroy ###");
        String key = test + ": DETECT_LEAK_CONNECTION_AND_DESTROY";
        try {
            println("Thread Sleep(s): " + connectionLeakTimeout);
            Thread.sleep(connectionLeakTimeout * 1000);
            for (int i = NumOfCon - 1; i >= 0; i--) {
                processEvent(EVENT.DESTROY_LEAK_CONNECTION);
            }
            if (tran) {
                validate(key, original);
            } else {
                validate(key, getExpectedMonitorData());
            }

        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public void tearDown() {
        stat.printSummary("jdbc-pooling-monitorID");
        remote.ejbRemove();
    }

    private void validate(String testKey, JDBCMonitorData expectedMonitorData) throws IOException {
        JDBCMonitorData actualResult = rest.monitor();
        report(actualResult, expectedMonitorData);
        stat.addStatus(testKey, expectedMonitorData.compare(actualResult) ? stat.PASS : stat.FAIL);
    }

    private void processEvent(EVENT event) {
        JDBCMonitorData expectedPrevData = getExpectedMonitorData();
        boolean max = false;

        switch (event) {
            case GET_CONNECTION:
                if (pooling) {
                    if (expectedPrevData.numconnfree.get(Constants.CURRENT) == 0) {
                        int numconnspace = maxPoolSize - (expectedPrevData.numconncreated - expectedPrevData.numconndestroyed);
                        if (numconnspace > 0) {
                            int addValue = (numconnspace > this.resize) ? this.resize : numconnspace - this.resize;
                            expectedPrevData.numconncreated += addValue;
                            expectedPrevData.increment(expectedPrevData.numconnfree, addValue);
                        } else {
                            max = true;
                        }
                    }
                } else {
                    expectedPrevData.numconncreated++;
                    expectedPrevData.increment(expectedPrevData.numconnfree);
                }

                if (!max) {
                    expectedPrevData.decrement(expectedPrevData.numconnfree);
                    expectedPrevData.increment(expectedPrevData.numconnused);
                    expectedPrevData.numconnacqired++;
                }
                break;

            case GET_SHARED_CONNECTION:
                expectedPrevData.numconnacqired++;
                break;

            case CLOSE_CONNECTION:
                expectedPrevData.decrement(expectedPrevData.numconnused);
                if (pooling) {
                    expectedPrevData.increment(expectedPrevData.numconnfree);
                } else {
                    expectedPrevData.numconndestroyed++;
                }
                expectedPrevData.numconnreleased++;
                break;

            case CLOSE_SHARED_CONNECTION:
                expectedPrevData.numconnreleased++;
                break;

            case MARK_CONNECTION_AS_BAD:
                expectedPrevData.decrement(expectedPrevData.numconnused);
                expectedPrevData.numconndestroyed++;
                expectedPrevData.numconnreleased++;
                break;

            case DESTROY_LEAK_CONNECTION:
                expectedPrevData.numconnpotentialconnleak++;
                expectedPrevData.decrement(expectedPrevData.numconnused);
                expectedPrevData.numconndestroyed++;
                break;
        }
    }

    private synchronized JDBCMonitorData getExpectedMonitorData() {
        if (expectedMonitorData == null) {
            expectedMonitorData = new JDBCMonitorData();
            expectedMonitorData.reset(this.pooling, this.minPoolSize);
        }
        return expectedMonitorData;
    }

    private void report(JDBCMonitorData actualResult, JDBCMonitorData expectedResult) {
        StringBuilder sb = new StringBuilder();
        sb.append("[expected] ");
        sb.append("numconnused = ");
        sb.append("curr:").append(expectedResult.numconnused.get(Constants.CURRENT));
        sb.append(", high:").append(expectedResult.numconnused.get(Constants.HIGHWATERMARK));
        sb.append(", low:").append(expectedResult.numconnused.get(Constants.LOWWATERMARK)).append("\n");
        sb.append("[  actual] ");
        sb.append("numconnused = ");
        sb.append("curr:").append(actualResult.numconnused.get(Constants.CURRENT));
        sb.append(", high:").append(actualResult.numconnused.get(Constants.HIGHWATERMARK));
        sb.append(", low:").append(actualResult.numconnused.get(Constants.LOWWATERMARK)).append("\n");

        sb.append("[expected] ");
        sb.append("numconnfree = ");
        sb.append("curr:").append(expectedResult.numconnfree.get(Constants.CURRENT));
        sb.append(", high:").append(expectedResult.numconnfree.get(Constants.HIGHWATERMARK));
        sb.append(", low:").append(expectedResult.numconnfree.get(Constants.LOWWATERMARK)).append("\n");
        sb.append("[  actual] ");
        sb.append("numconnfree = ");
        sb.append("curr:").append(actualResult.numconnfree.get(Constants.CURRENT));
        sb.append(", high:").append(actualResult.numconnfree.get(Constants.HIGHWATERMARK));
        sb.append(", low:").append(actualResult.numconnfree.get(Constants.LOWWATERMARK)).append("\n");

        sb.append("[expected] ");
        sb.append("numconncreated = ").append(expectedResult.numconncreated).append("\n");
        sb.append("[  actual] ");
        sb.append("numconncreated = ").append(actualResult.numconncreated).append("\n");

        sb.append("[expected] ");
        sb.append("numconndestroyed = ").append(expectedResult.numconndestroyed).append("\n");
        sb.append("[  actual] ");
        sb.append("numconndestroyed = ").append(actualResult.numconndestroyed).append("\n");

        sb.append("[expected] ");
        sb.append("numconnacqired = ").append(expectedResult.numconnacqired).append("\n");
        sb.append("[  actual] ");
        sb.append("numconnacqired = ").append(actualResult.numconnacqired).append("\n");

        sb.append("[expected] ");
        sb.append("numconnreleased = ").append(expectedResult.numconnreleased).append("\n");
        sb.append("[  actual] ");
        sb.append("numconnreleased = ").append(actualResult.numconnreleased).append("\n");

        sb.append("[expected] ");
        sb.append("numconnpotentialconnleak = ").append(expectedResult.numconnpotentialconnleak).append("\n");
        sb.append("[  actual] ");
        sb.append("numconnpotentialconnleak = ").append(actualResult.numconnpotentialconnleak).append("\n");

        println(sb.toString());
    }

    public void print(String str) {
        if (debug) {
            System.out.print(str);
        }
    }

    public void println(String str) {
        if (debug) {
            System.out.println(str);


        }
    }
}

enum EVENT {

    GET_CONNECTION, GET_SHARED_CONNECTION, CLOSE_CONNECTION, CLOSE_SHARED_CONNECTION, MARK_CONNECTION_AS_BAD, DESTROY_LEAK_CONNECTION
}