package org.glassfish.devtest.pooling.monitor.client.utils;

import com.sun.jersey.api.client.Client;
import com.sun.jersey.api.client.ClientResponse;
import com.sun.jersey.api.client.WebResource;
import java.io.IOException;
import java.util.Map;
import javax.ws.rs.core.MediaType;
import org.codehaus.jackson.map.ObjectMapper;

public class RestHelper {

    Client client = null;
    String poolName = "";

    public RestHelper(String poolName) {
        this.poolName = poolName;
        client = Client.create();
    }

    public void updatePoolSetting(String updateParam, boolean debug) throws IOException {
        post(updateParam);
        if(debug){
            ClientResponse response = get(true);
            Map entity = getEntity(response);
            System.out.println("updatePoolSetting: "+entity);
        }
    }

    public JDBCMonitorData monitor() throws IOException {
        ClientResponse response = get(false);
        Map entity = getEntity(response);
        JDBCMonitorData data = createJDBCMonitorData(entity);
        //System.out.println(data);
        return data;
    }

    public void post(String updateParam) {
        WebResource webResource = client.resource(Constants.MANAGEMENT_URL + this.poolName + "/");
        webResource.type(MediaType.APPLICATION_FORM_URLENCODED).post(ClientResponse.class, updateParam);
    }

    public ClientResponse get(boolean management) {
        String baseURL = null;
        if (management) {
            baseURL = Constants.MANAGEMENT_URL + this.poolName + "/";
        } else {
            baseURL = Constants.MONITORING_URL + this.poolName + "/";
        }
        WebResource webResource = client.resource(baseURL);
        return webResource.accept(MediaType.APPLICATION_JSON).get(ClientResponse.class);
    }

    private Map getEntity(ClientResponse response) throws IOException {
        ObjectMapper mapper = new ObjectMapper();
        Map jsonMap = mapper.readValue(response.getEntity(String.class), Map.class);
        Map extraProperties = (Map)jsonMap.get(Constants.EXTRA_PROPERTIES);
        return (Map) extraProperties.get(Constants.ENTITY);
    }

    private JDBCMonitorData createJDBCMonitorData(Map entity) {
        JDBCMonitorData data = new JDBCMonitorData();
        data.numconnused.put(Constants.CURRENT, retrieve(entity, Constants.NUMCONNUSED, Constants.CURRENT));
        data.numconnused.put(Constants.HIGHWATERMARK, retrieve(entity, Constants.NUMCONNUSED, Constants.HIGHWATERMARK));
        data.numconnused.put(Constants.LOWWATERMARK, retrieve(entity, Constants.NUMCONNUSED, Constants.LOWWATERMARK));

        data.numconnfree.put(Constants.CURRENT, retrieve(entity, Constants.NUMCONNFREE, Constants.CURRENT));
        data.numconnfree.put(Constants.HIGHWATERMARK, retrieve(entity, Constants.NUMCONNFREE, Constants.HIGHWATERMARK));
        data.numconnfree.put(Constants.LOWWATERMARK, retrieve(entity, Constants.NUMCONNFREE, Constants.LOWWATERMARK));

        data.numconncreated = retrieve(entity, Constants.NUMCONNCREATED, Constants.COUNT);
        data.numconndestroyed = retrieve(entity, Constants.NUMCONNDESTROYED, Constants.COUNT);

        data.numconnacqired = retrieve(entity, Constants.NUMCONNACQUIRED, Constants.COUNT);
        data.numconnreleased = retrieve(entity, Constants.NUMCONNRELEASED, Constants.COUNT);

        data.numconnpotentialconnleak = retrieve(entity, Constants.NUMCONNPOTENTIALCONNLEAK, Constants.COUNT);
        return data;
    }

    private int retrieve(Map entity, String label, String mark){
        Map record = ((Map) entity.get(label));
        int data = (Integer)record.get(mark);
        return data;
    }
}
