package org.glassfish.devtest.pooling.monitor.ejb;

import java.rmi.RemoteException;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.annotation.Resource;
import javax.ejb.Remove;
import javax.ejb.Stateful;
import javax.ejb.TransactionManagement;
import javax.ejb.TransactionManagementType;
import javax.sql.DataSource;
import javax.transaction.UserTransaction;

@TransactionManagement(TransactionManagementType.BEAN)
@Stateful
public class SimpleSessionBean implements SimpleSessionBeanRemote, SimpleSessionBeanLocal {

    @Resource(name = "jdbc/sample")
    private DataSource ds;
    private ArrayList<Connection> resources;
    @Resource
    private UserTransaction ut;

    @PostConstruct
    public void init() {
        System.out.println("SimpleSessionBean::init()");
        resources = new ArrayList<Connection>(20);
    }

    @PreDestroy
    public void destroy() {
        System.out.println("SimpleSessionBean::destroy()");
        try {
            closeAll();
        } catch (RemoteException ex) {
            ex.printStackTrace();
        }
        ds = null;
        resources = null;
    }

    @Remove
    public void ejbRemove() {
        System.out.println("SimpleSessionBean::ejbRemove()");
    }

    @Override
    public void getConnection() throws RemoteException {
        Connection con = null;
        try {
            con = ds.getConnection();
        } catch (Exception ex) {
            throw new RemoteException("SimpleSessionBean::getConnection()", ex);
        }
        if (con != null) {
            resources.add(con);
        }
    }

    @Override
    public boolean isSharedConnections() throws RemoteException {
        boolean result = true;
        try {
            Connection con1 = resources.get(0);
            Connection mc1 = ((com.sun.appserv.jdbc.DataSource) ds).getConnection(con1);
            for (int i = 1; i < resources.size(); i++) {
                Connection con2 = resources.get(i);
                Connection mc2 = ((com.sun.appserv.jdbc.DataSource) ds).getConnection(con2);
                result &= (mc1 == mc2);
            }
        } catch (Exception ex) {
            throw new RemoteException("SimpleSessionBean::isSharedConnections()", ex);
        }
        return result;
    }

    @Override
    public void close(int index) throws RemoteException {
        try {
            Connection con = resources.remove(index);
            con.close();
        } catch (Exception ex) {
            throw new RemoteException("SimpleSessionBean::close()", ex);
        }
    }

    public void closeAll() throws RemoteException {
        Iterator iter = resources.iterator();
        while (iter.hasNext()) {
            try {
                ((Connection) iter.next()).close();
            } catch (Exception ex) {
                throw new RemoteException("SimpleSessionBean::closeAll()", ex);
            }
        }
    }

    @Override
    public void markConnectionAsBad(int index) throws RemoteException {
        try {
            Connection con = resources.get(index);
            ((com.sun.appserv.jdbc.DataSource) ds).markConnectionAsBad(con);
        } catch (Exception ex) {
            throw new RemoteException("SimpleSessionBean::markConnectionAsBad()", ex);
        }
    }

    @Override
    public void beginForUT() throws RemoteException {
        try {
            ut.begin();
        } catch (Exception ex) {
            throw new RemoteException("SimpleSessionBean::beginForUT()", ex);
        }
    }

    @Override
    public void commitForUT() throws RemoteException {
        try {
            ut.commit();
        } catch (Exception ex) {
            throw new RemoteException("SimpleSessionBean::commitForUT()", ex);
        }
    }

    @Override
    public void rollbackForUT() throws RemoteException {
        try {
            ut.rollback();
        } catch (Exception ex) {
            throw new RemoteException("SimpleSessionBean::rollbackForUT()", ex);
        }
    }

    @Override
    public void commitForConnection(int index) throws RemoteException {
        try {
            Connection con = resources.get(index);
            con.commit();
        } catch (Exception ex) {
            throw new RemoteException("SimpleSessionBean::commitForConnection()", ex);
        }
    }

    @Override
    public void rollbackForConnection(int index) throws RemoteException {
        try {
            Connection con = resources.get(index);
            con.rollback();
        } catch (Exception ex) {
            throw new RemoteException("SimpleSessionBean::rollbackForConnection()", ex);
        }
    }
}
